@@ -63,9 +63,11 @@ class APIRequestor:
         return resp
 
     def request_headers(self, api_key):
-        user_agent = "Circle Python SDK"
+        referer = "https://getlinus.io"
+        user_agent = "Circle Python SDK"  # "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.142 Safari/537.36"
         headers = {
             "Authorization": "Bearer %s" % api_key,
+            "Referer": referer,
             "User-Agent": user_agent,
             "Content-Type": "application/json",
             "Accept": "application/json",
@@ -111,7 +113,8 @@ class APIRequestor:
             "delete",
         ):
             post_data = None
-            # TODO: generate query params for get/delete methods
+            if params:
+                abs_url = abs_url + "?" + urllib.parse.urlencode(params)
         else:
             raise error.APIConnectionError(
                 f"Invalid HTTP method, {method}."
