@@ -7,7 +7,9 @@ class CircleResponse:
         self.code = code
         self.headers = headers
 
-        self.data = json.loads(body)["data"]
+        json_body = json.loads(body)
+        self.data = json_body.get("data", json_body)
+        print(self.data)
 
     @property
     def idempotency_key(self):
