@@ -1,10 +1,15 @@
 import time
 from circle import error
+import threading
 import requests
+import logging
+from requests.adapters import HTTPAdapter
+from requests.packages.urllib3.util.retry import Retry
+
+logging.basicConfig(level=logging.DEBUG)
 
 import textwrap
 
-
 def new_default_http_client(*args, **kwargs):
     if requests:
         return RequestsClient(*args, **kwargs)
@@ -13,11 +18,13 @@ def new_default_http_client(*args, **kwargs):
             "Requests must be installed as it is currently the only option for an HTTP client."
         )
 
-
 class HttpClient:
     MAX_DELAY = 2  # in seconds
     INITIAL_DELAY = 0.5  # in seconds
 
+    def __init__(self):
+        self._thread_local = threading.local()
+
     def request_with_retries(self, method, url, headers, post_data=None, proxies=None):
         num_retries = 0
         while True:
@@ -83,17 +90,32 @@ class RequestsClient(HttpClient):
 
     def request(self, method, url, headers, post_data=None, proxies=None):
 
-        if not self._session:
-            self._session = requests.Session()
+        if getattr(self._thread_local, "session", None) is None:
+            self._thread_local.session = self._session or requests.Session()
+            self._thread_local.session.keep_alive = False # TODO: remove this for performance improvement
+
+            retry = Retry(
+                total=self._max_network_retries,
+                connect=5,
+                backoff_factor=0.1
+            )
+
+            adapter = HTTPAdapter(
+                max_retries=retry
+            )
+
+            self._thread_local.session.mount('http://', adapter)
+            self._thread_local.session.mount('https://', adapter)
 
         try:
-            res = self._session.request(
+            res = self._thread_local.session.request(
                 method,
                 url,
                 headers=headers,
                 data=post_data,
                 timeout=self._timeout,
                 proxies=proxies,
+                verify=True, # TODO: if all else fails, set this to False
             )
         except Exception as e:
             self._handle_request_error(e)
@@ -103,7 +125,15 @@ class RequestsClient(HttpClient):
     def _handle_request_error(self, e):
         """"""
         if isinstance(e, requests.exceptions.ConnectionError):
-            msg = "Unexpected connection error communicating with circle. "
+            msg = "Request [Connection Error] detected communicating with [Circle API]"
+            err = f"{type(e).__name__}: {str(e)}"
+            should_retry = True
+        if isinstance(e, requests.Timeout):
+            msg = "Request [Timeout] detected communicating with [Circle API]"
+            err = f"{type(e).__name__}: {str(e)}"
+            should_retry = True
+        if isinstance(e, requests.RequestException):
+            msg = "Request [Exception] detected communicating with [Circle API]"
             err = f"{type(e).__name__}: {str(e)}"
             should_retry = True
         else:
